@@ -1,89 +1,43 @@
+# $Id: /mirror/coderepos/lang/perl/MooseX-WithCache/trunk/lib/MooseX/WithCache.pm 88178 2008-10-16T07:50:55.595631Z daisuke  $
 
 package MooseX::WithCache;
-use MooseX::Role::Parameterized;
-use 5.008;
-use constant DEBUG => $ENV{MOOSEX_WITHCACHE_DEBUG} ? 1 : 0;
-our $VERSION   = '0.00999_04';
-our $AUTHORITY = 'cpan:DMAKI';
-my %BACKENDS;
-
-# This is solely for backwards compatibility
+use Moose;
 use Moose::Exporter;
+use 5.008;
+our $VERSION   = '0.00001';
+our $AUTHORITY = 'cpan:DMAKI';
+
 Moose::Exporter->setup_import_methods(
-    with_caler => [ 'with_cache' ],
+    with_caller => [ 'with_cache' ]
 );
 
+no Moose;
+no Moose::Exporter;
+
 sub with_cache {
     my ($caller, $name, %args) = @_;
 
-    Carp::carp("use of with_cache for MooseX::WithCache is now deprecated. Use parameterized roles directly");
-
-    Moose::Util::apply_all_roles(
-        $caller,
-        __PACKAGE__, {
-            %args,
-            name => $name,
-        }
-    );
-}
-
-parameter backend => (
-    isa => 'Str',
-    required => 1,
-    default => 'Cache::Memcached',
-);
-
-parameter name => (
-    isa => 'Str',
-    required => 1,
-    default => 'cache'
-);
-
-role {
-    my $p = shift;
-
-    my $name          = $p->name;
-    my $backend_class = $p->backend;
-
-    if ($backend_class !~ s/^\+//) {
-        $backend_class = "MooseX::WithCache::Backend::$backend_class";
-    }
-    Class::MOP::load_class($backend_class);
-    my $backend = $BACKENDS{ $backend_class };
-    if (! $backend ) {
-        $backend = $backend_class->new();
-        $BACKENDS{ $backend_class } = $backend;
-    }
-
-    has $name => (
-        is => 'rw',
-        isa => $backend->cache_type(),
-        coerce => $backend->can_coerce(),
-    );
-
-    has cache_disabled => (
-        is => 'rw',
-        isa => 'Bool',
-        default => 0
-    );
+    my $meta = $caller->meta;
 
     # key generator generates the appropriate cache key from given key(s). 
-    has cache_key_generator => (
-        is      => 'rw',
-        does    => 'MooseX::WithCache::KeyGenerator',
+    $meta->add_attribute(
+        key_generator => (
+            is   => 'rw',
+            does => 'MooseX::WithCache::KeyGenerator',
+        )
     );
 
-    method __get_cache => sub { $_[0]->$name };
-    method cache_debug => sub {
-        my $self = shift;
-        print STDERR "[CACHE]: @_\n";
-    };
+    # backend is the actual cache backend. this is the main guy that
+    # does the cache-aware  meta protocol munging.
+    # we
+    my $backend = $args{backend} || 'Cache::Memcached';
+    my $module  = "MooseX::WithCache::Backend::$backend";
 
-    my $methods = $backend->methods();
-    while (my($method, $code) = each %$methods) {
-        method $method, $code;
-    }
-};
+    Class::MOP::load_class($module);
+    $module->new(
+        cache_name => $name,
+    )->setup($caller, %args);
+}
 
 1;
 
@@ -99,7 +53,7 @@ MooseX::WithCache - Easy Cache Access From Moose Objects
     use Moose;
     use MooseX::WithCache;
 
-    with 'MooseX::WithCache' => {
+    with_cache 'cache' => (
         backend => 'Cache::Memcached',
     );
 
@@ -123,13 +77,7 @@ MooseX::WithCache - Easy Cache Access From Moose Objects
     my $foo = $object->get_foo();
 
     # if you want to do something with the cache object,
-    # you can access it via the name you gave in with staemtent
-    # 
-    # with 'MooseX::WithCache' => {
-    #    name => 'cache', # default
-    #    ....
-    # }
-
+    # you can access it via the name you gave in with_cache
     my $cache = $object->cache;
 
 =head1 DESCRIPTION
@@ -164,7 +112,7 @@ the cache_disabled() attribute that gets installed
 =head2 DEBUG OUTPUT
 
 You can inspect what's going on with respect to the cache, if you specify
-MOOSEX_WITHCACHE_DEBUG=1 in the environment. This will caue MooseX::WithCache to
+MOOSEX_CACHE_DEBUG=1 in the environment. This will caue MooseX::WithCache to
 display messages to STDERR.
 
 =head2 KEY GENERATION
@@ -172,8 +120,8 @@ display messages to STDERR.
 Sometimes you want to give compound keys, or simply transform the cache keys
 somehow to normalize them.
 
-MooseX::WithCache supports this through the cache_key_generator attribute.
-The cache_key_generator simply needs to be a MooseX::WithCache::KeyGenerator
+MooseX::WithCache supports this through the key_generator attribute.
+The key_generator simply needs to be a MooseX::WithCache::KeyGenerator
 instance, which accepts whatever key provided, and returns a new key.
 
 For example, if you want to provide complex key that is a perl structure,
@@ -184,9 +132,15 @@ Simply specify it in the constructor:
 
     MyObject->new(
         cache => ...,
-        cache_key_generator => MooseX::WithCache::KeyGenerator::DumpChecksum->new()
+        key_generator => MooseX::WithCache::KeyGenerator::DumpChecksum->new()
     );
 
+=head1 METHODS
+
+=head2 with_cache($name, %opts)
+
+Configures the cache for the object.
+
 =head1 AUTHOR
 
 Daisuke Maki C<< <daisuke@endeworks.jp> >>
